****************************************************************************
*
*                           M A I N P L T S . P R G
*
*  AUTHOR: Richard A. Schummer            October 1993
*
*  COPYRIGHT (c) 1993, 1994    Richard A. Schummer
*     38947 Trawick Ct.  
*     Sterling Heights, MI  48310-1771
*     CompuServe Id # 70254,1643
*
*     Free for the use by all FoxPro developers around the world!
*
*  PROGRAM DESCRIPTION: 
*     This utility provides a FoxPro developer with a listing of all the 
*     different objects in a selected project (screens, reports, labels, 
*     programs, queries, menus, files, libraries, etc.).  
*
*     This program was designed to provide a checklist for the developer when 
*     they needed to process all the items in a project.  It also provides a very 
*     basic start to some technical documentation for the project.  It quickly 
*     evolved to provide detail maintained for the objects and the timestamp stored
*     in the project.  It provides good information in case you need to rebuild
*     a corrupted project file from scratch.  The screen settings stored in 
*     project manager are available on this output and can be a life saver
*     when rebuilding since all the screen "settings" (define and release window, 
*     modal, open and close databases, etc.) are saved in a project.
*
*     Several options allow the user to print to the screen or to a printer, 
*     and determine what type of data is included.  The program is cross-platform 
*     for DOS and Windows and Macintosh.
*
*     If no parameters are sent then a screen is displayed for the user to select
*     the different options available.
*
*     The report is displayed on the screen (if selected) in the print preview mode
*     that is provided by FoxPro.  This means that DOS based executions are limited
*     to forward progress through the report (sorry DOS users).  The print preview
*     (and many other feature such as report print quality) are much improved in
*     the Windows and Macintosh platforms.
*
*  CALLING SYNTAX: 
*     DO MainPlts WITH <cOutPut>,<lTimeStamp>,<lNotes>,<lDirectories>,<lFullDetail>
*
*     Sample:
*     DO MainPlts WITH "PRINTER", .T., .T., .T., .F.
* 
*  INPUT PARAMETERS: 
*     (All are optional, if no parameters are sent, a screen to select 
*      options is displayed)
*
*     cOutput     = Character, not required (defaults to "PRINTER"), 
*                    "PRINTER" sends the report to the default printer
*                    "SCREEN"  sends the report to the REPORT FORM PRINT PREVIEW
*                    "BROWSE"  sends the output to a cursor which is browseable
*                    "FILE"    sends the report to a file selected by the user
*     lTimeStamp  = Logical, not required (defaults to .T.),
*                    .T. prints date and time for project objects
*                    .F. bypasses printing of date/time on the report
*     lNotes      = Logical, not required (defaults to .T.),
*                    .T. prints note details for project and screens
*                    .F. bypasses printing of note details on the report
*     lDirectories= Logical, not required (defaults to .T.),
*                    .T. prints directories for all objects except .SPRs
*                    .F. bypasses printing of directories on the report
*     lFullDetail = Logical, not required (defaults to .F.),
*                    .T. prints all note details for screens, verbage printed
*                        indicating options set on or off
*                    .F. prints partial note details for screens, verbage 
*                        printed indicating only the options set ON
*
*            NOTE : If you need one of the option parameters, you must use
*                   all the parameters that proceed the parameter you want.
*
*  OUTPUT PARAMETERS:
*     None
*
*  DATABASES ACCESSED: 
*     Project file selected by the user
* 
*  GLOBAL VARIABLES REQUIRED:
*     None
*
*  GLOBAL PROCEDURES REQUIRED:
*     None
* 
*  CODING STANDARDS:
*     Version 1.0 compliant with no exceptions                             
*  
*  TEST INFORMATION:
*     None
*   
*  SPECIAL REQUIREMENTS/DEVICES:
*     Printer if option is used.
*
*  FUTURE ENHANCEMENTS:
*     1) Select the object type(s) that you want to list from a project
*     2) Second report that lists objects in name order
*     3) Add icon graphics for each object type (Windows/Mac platform)
*
*  LANGUAGE/VERSION:
*     FoxPro  2.5  or higher                                                  
* 
****************************************************************************
*
*                        C H A N G E    L O G
*
*   Date              SE            Version          Description
* --------  ----------------------  -------  -----------------------------
* 10/28/93  Richard A. Schummer     1.0      Created Program
* ------------------------------------------------------------------------
* 03/22/94  Richard A. Schummer     1.1      Added timestamp information,
*                                            and ability to print partial
*                                            details on report.
* ------------------------------------------------------------------------
* 04/26/94  Richard A. Schummer     1.1      Added directories to report
* ------------------------------------------------------------------------
* 05/17/94  Richard A. Schummer     1.2      Added screen for parameter 
*                                            selection
* ------------------------------------------------------------------------
*
****************************************************************************

PARAMETER cOutput, lTimeStamp, lNotes, lDirectory, lFullDetail

PRIVATE LIKE cOld*                     &&  Saves SETtings for restoration later

PRIVATE cProjDbf                       &&  Holds name of project database processed

PRIVATE cOutPutRb                      &&  Contains the destination of the output
PRIVATE cNotesRb                       &&  Contains the detail level of notes reported
PRIVATE cOkPb                          &&  Button holding user selected OK/Cancel
PRIVATE cAboutPb                       &&  Button that activates About screen

PRIVATE nSubDir                        &&  Number of relative subdirectories in file name
PRIVATE nIndex                         &&  Used as SUBSTR index to parse file names
PRIVATE cParentDir                     &&  Parent directory name for file

PRIVATE cVersion                       &&  Project Lister version number 

SET TALK OFF

cProjDbf  = SPACE(90)
cOutPutRb = "To printer"
cNotesRb  = "Partial details"
cOkPb     = "CANCEL"
cAboutPb  = ""
cVersion  = "1.2"

PUSH KEY CLEAR                         &&  Clean the ON KEY slate

*  Handle option parameters

DO CASE
   CASE PARAMETERS() = 0
      cOutput     = "PRINTER"
      lTimeStamp  = .T.
      lNotes      = .T.
      lDirectory  = .T.
      lFullDetail = .F.

      ON KEY LABEL F1 DO HelpPR WITH VARREAD()
      
      DO ParmPLts.spr                  &&  Provide user with option selection screen

      IF UPPER(cOkPb) = "CANCEL"
          DO CleanUpPR                 &&  Clean up the environment (green programmer <g>)
          RETURN
      ENDIF

   CASE PARAMETERS() = 1
      lTimeStamp  = .T.
      lNotes      = .T.
      lDirectory  = .T.
      lFullDetail = .F.

   CASE PARAMETERS() = 1
      lNotes      = .T.
      lDirectory  = .T.
      lFullDetail = .F.

   CASE PARAMETERS() = 3
      lDirectory  = .T.
      lFullDetail = .F.

   CASE PARAMETERS() = 4
      lFullDetail = .F.
      
ENDCASE


* Verify parameters are correct data type

IF TYPE('cOutput') != "C"
   WAIT WINDOW "Output directional parameter not character value"
   DO CleanUpPR                        &&  Clean up the environment (green programmer <g>)
   RETURN
ENDIF

IF TYPE('lTimeStamp') != "L"
   WAIT WINDOW "Timestamp parameter not logical value"
   DO CleanUpPR                        &&  Clean up the environment (green programmer <g>)
   RETURN
ENDIF

IF TYPE('lNotes') != "L"
   WAIT WINDOW "Notes printing parameter not logical value"
   DO CleanUpPR                        &&  Clean up the environment (green programmer <g>)
   RETURN
ENDIF

IF TYPE('lDirectory') != "L"
   WAIT WINDOW "Directory printing parameter not logical value"
   DO CleanUpPR                        &&  Clean up the environment (green programmer <g>)
   RETURN
ENDIF

IF TYPE('lFullDetail') != "L"
   WAIT WINDOW "Detail selection parameter not logical value"
   DO CleanUpPR                        &&  Clean up the environment (green programmer <g>)
   RETURN
ENDIF

IF VAL(SUBSTR(VERSION(),AT(".",VERSION())-1, 3)) < 2.5
   WAIT WINDOW "This program requires FoxPro 2.5 or higher"
   DO CleanUpPR                        &&  Clean up the environment (green programmer <g>)
   RETURN
ENDIF

* Close databases if open from crash of a previous run of this program
* (which should not normally happen, but you know how things happen), or
* existing environment has a database opened that has alias of one used
* in this program.

IF USED('project')
   SELECT project
   USE
ENDIF

IF USED('query')
   SELECT query
   USE
ENDIF

IF USED('projlist')
   SELECT projlist
   USE
ENDIF

IF USED('projrep')
   SELECT projrep
   USE
ENDIF


* Save old SETtings, set needed environment 

cOldExclusive = SET("EXCLUSIVE")
cOldDeleted   = SET("DELETED")

SET EXCLUSIVE OFF
SET DELETED ON


* Get the project file to be reported on if not picked using parameter screen

IF EMPTY(cProjDbf)
   DO vSelProjPbPR                     &&  Allow the user to select the project file
ENDIF


* Check to see if project file was selected and make sure it does exist

IF EMPTY(cProjDbf)
   DO CleanUpPR                        &&  Clean up the environment before leaving app (green programmer <g>)
   RETURN
ENDIF

IF !FILE(cProjDbf)
   WAIT WINDOW "Project file not found..."
   DO CleanUpPR                        &&  Clean up the environment before leaving app (green programmer <g>)
   RETURN
ENDIF


* Add "calculated" fields to cursor which is reported on

WAIT WINDOW "Collecting project information..." NOWAIT

* Makes file read-only, protects from corruption

USE (cProjDbf) AGAIN NOUPDATE ALIAS project   

* Collect information about the project file

SELECT * ;
   FROM project ;
   WHERE UPPER(type) = "H" ;
   INTO CURSOR query


* Initialize home directory variable

IF !EOF()
   cHomeDir = ALLTRIM(HomeDir) + IIF(RAT("\", HomeDir) != LEN(ALLTRIM(HomeDir)), "\","")
   cHomeDir = UPPER(STRTRAN(cHomeDir, CHR(0), ""))
ELSE
   cHomeDir = ""
ENDIF

USE                                    &&  Close temporary cursor query
SELECT project

SET EXCLUSIVE ON

nRows = AFIELDS(ac_Stru)

DIMENSION ac_Stru(nRows+3,4)           &&  Contains table definition information

ac_Stru(nRows+1,1) = "cShortName"      &&  Add ShortName field to newly created cursor
ac_Stru(nRows+1,2) = "C"
ac_Stru(nRows+1,3) = 12
ac_Stru(nRows+1,4) = 0

ac_Stru(nRows+2,1) = "mNotes"          &&  Add mNotes field to newly created cursor
ac_Stru(nRows+2,2) = "M"
ac_Stru(nRows+2,3) = 10
ac_Stru(nRows+2,4) = 0

ac_Stru(nRows+3,1) = "mDirName"        &&  Add mDirName field to newly created cursor
ac_Stru(nRows+3,2) = "M"
ac_Stru(nRows+3,3) = 10
ac_Stru(nRows+3,4) = 0

CREATE CURSOR projlist FROM ARRAY ac_Stru


* Collect initial information from project, add on "calculated" fields

SELECT * ;
   FROM project ;
   ORDER BY    type ;
   INTO CURSOR query

WAIT WINDOW "Processing project information..." NOWAIT

SELECT project                          &&  Close the project file
USE

SELECT query

SCAN
   SCATTER MEMVAR MEMO

   * Count Subdirectories so actual directory name can be printed
      
   nSubDir = 0
   nIndex  = 1
   
   DO WHILE .T.
      DO CASE
         CASE SUBSTR(name, nIndex, 4) = "..\\"
            nSubDir = nSubDir + 1
            nIndex  = nIndex + 4
         CASE SUBSTR(name, nIndex, 3) = "..\"
            nSubDir = nSubDir + 1
            nIndex  = nIndex + 3
         OTHERWISE
            EXIT
      ENDCASE
   ENDDO
   
   * Process "calculated" fields, remember mNotes is based on parameter
   
   DO CASE
      CASE nSubDir = 0 AND SUBSTR(name,1,1) = "\"     &&  Absolute directories
         cParentDir = SUBSTR(cHomeDir, 1, 2)
      CASE nSubDir = 0 AND SUBSTR(name,2,1) = ":"     &&  Complete path
         cParentDir = ""
      CASE nSubDir = 0 AND SUBSTR(name,1,1) != "\"    &&  Relative directories
         cParentDir = cHomeDir
      OTHERWISE                                       && One or more parental directories
         cParentDir = UPPER(SUBSTR(cHomeDir, 1, RAT("\",cHomeDir, nSubDir+1)))
   ENDCASE
   
   DO CASE
      CASE UPPER(Type) = "H"           &&  Project directory is home directory
         m.mDirName =  cHomeDir
      CASE Type = "S"                  &&  Screen programs are not actual directories, only SPR name
         m.mDirName =  ""
      OTHERWISE
         m.mDirName = cParentDir + UPPER(SUBSTR(name, nIndex , RAT("\",name)-nIndex+1))
   ENDCASE
   
   m.cShortName = ShortNameFN()
   m.mNotes     = MemoCreateFN()
   
   
   INSERT INTO projlist FROM MEMVAR
ENDSCAN


* Sort the cursor in the order needed for the report

SELECT * ;
   FROM projlist ;
   ORDER BY    Type, cShortName ;
   INTO CURSOR projrep

SELECT projrep

WAIT CLEAR


* Output the report to screen, file or printer (determined from parameter sent by user)

DO CASE 
   CASE cOutput = "PRINTER"
      IF _DOS OR _UNIX
         WAIT WINDOW "Printing project report..." NOWAIT
      ENDIF   
   
      REPORT FORM Pj01Pltr TO PRINTER NOCONSOLE
 
   CASE cOutput = "FILE"
      cFileName = PUTFILE("Report file name","PROJLIST.TXT","TXT;REP","Select")

      IF !EMPTY(cFileName)
         cOldPdSetup = SET("PDSETUP")
         SET PDSETUP TO             &&  No print driver used for file output
         
         REPORT FORM Pj01Pltr ;
            TO FILE (cFileName) ;
            NOCONSOLE

         IF _DOS or _UNIX
            SET PDSETUP TO "&cOldPdSetup"
         ENDIF
      ELSE
         WAIT WINDOW "No output file selected, no file output created" NOWAIT
      ENDIF

   CASE cOutput = "BROWSE"
      DEFINE WINDOW wbBrowser FROM 1,0 TO SROWS()-1, SCOLS()-1 ;
         FONT "Arial", 8 ;
         GROW CLOSE DOUBLE FLOAT ZOOM

      cBrowseDate = IIF(lTimeStamp, ', cDateTime = TStamp2DateFN(timestamp,"DATETIME") :R:H="Date and Time" ', " ")
      cBrowseDate = IIF((lNotes OR lDirectory) AND !EMPTY(cBrowseDate), cBrowseDate + ",", cBrowseDate)
      cBrowseNote = IIF(lNotes OR lDirectory, IIF(EMPTY(cBrowseDate),",","")+'mNotes :R:H="Notes"', " ")
      
      BROWSE LAST ;
         FIELDS cIndicator = IIF(mainprog,"*",IIF(exclude,""," ")) :R:H=" ", ;
                cObjType   = ObjectTypeFN() :R:H="Type", ;
                cShortName :R:H="Name" ;
                &cBrowseDate ;
                &cBrowseNote ;
         TITLE "Project Lister Browse" ;
         NOAPPEND NODELETE ;
         WINDOW wbBrowser
         
      RELEASE WINDOW wbBrowser
      
   OTHERWISE
      REPORT FORM Pj01Pltr PREVIEW
ENDCASE

WAIT CLEAR

DO CleanUpPR                           &&  Clean up the environment before leaving app (green programmer <g>)

RETURN



****************************************************************************
*
*                              ObjectTypeFN
*
*  This procedure returns the discription of the object type based on the
*  code stored on the data record.  This list is based on FoxPro 2.5 and
*  the power tool Table Structure documentation found in Appendix A of the
*  FoxPro Developer's Guide.
*
*  INPUT PARAMETERS:
*     None
*
*  OUTPUT PARAMETERS:
*     cRetString = The description for the object being printed on the report.
*
****************************************************************************

FUNCTION ObjectTypeFN

PRIVATE cRetString                     &&  Literal returned to the calling program

DO CASE 
   CASE  Type = "B"
      cRetString = "Labels"
   CASE  Type = "D"
      cRetString = "Database Tables"
   CASE  Type = "F"
      cRetString = "Format Files"
   CASE  Type ="H"
      cRetString = "Project File"
   CASE  Type = "I"
      cRetString = "Index Files"
   CASE  Type = "L"
      cRetString = "Library Files"
   CASE  Type = "M"
      cRetString = "Menu Files"
   CASE  Type = "P"
      cRetString = "Program Files"
   CASE  Type = "Q"
      cRetString = "Queries"
   CASE  Type = "R"
      cRetString = "Report Files"
   CASE  Type = "s"
      cRetString = "Screen Sets"
   CASE  Type = "S"
      cRetString = "Screen Programs"
   CASE  Type = "x"
      cRetString = "Other Files"
   CASE  Type = "Z"
      cRetString = "Applications"
   OTHERWISE
      cRetString = "Unknown Type"
ENDCASE

RETURN cRetString   
   
   
   
****************************************************************************
*
*                              MemoCreateFN
*
*  This procedure assembles the m.Notes field based on the object type.
*  The breakout by object type within a project is necessary since 
*  specific fields are used depending on each different object type.
*  All the objects are CASEd even though they may not have any detail
*  to be printed.  This was done to document this fact and allow for future
*  enhancements (you never know what Microsoft might change).
*
*  The object type list is based on FoxPro 2.5 and the power tool Table 
*  Structure documentation found in Appendix A of the FoxPro Developer's Guide.
*
*  INPUT PARAMETERS:
*     None
*
*  OUTPUT PARAMETERS:
*     cRetString = The information collected for the object being printed 
*                  on the report.
*
****************************************************************************

FUNCTION MemoCreateFN

PRIVATE cRetString                     &&  Value returned by the function

cRetString = ""

IF lNotes 
   DO CASE 
      CASE  Type = "B"
      CASE  Type = "D"
      CASE  Type = "F"
      CASE  Type = "H"
         cRetString = IIF(Debug, "Debug code created, ", IIF(lFullDetail,"No debug code, ",""))+ ;
                      IIF(Encrypt, "App is encrypted, ",  ;
                                   IIF(lFullDetail,"App is not encrypted, ",""))+ ;
                      IIF(NoLogo, IIF(lFullDetail,"No FoxPro logo at start up, ",""), ;
                                  "Shows FoxPro logo at start up, ")+ ;
                      "Comment style is "+IIF(CmntStyle=0, "box, ", "asterick, ")+ ;
                      IIF(SaveCode, "Generated code is saved in " + ALLTRIM(STRTRAN(OutFile, CHR(0), "")) + ;
                                    " directory,","Generated code is not saved,")+ ;
                      IIF(EMPTY(DevInfo),"", ;
                          CHR(13)+ALLTRIM(STRTRAN(DevInfo, CHR(0), IIF(_DOS OR _UNIX,CHR(13),CHR(32)))))
      CASE  Type = "I"
      CASE  Type = "L"
      CASE  Type = "M"
      CASE  Type = "P"
      CASE  Type = "Q"
      CASE  Type = "R"
      CASE  Type = "S"
         cRetString = IIF(OpenFiles, "Files opened, ", ;
                                      IIF(lFullDetail,"Files not opened, ",""))+ ;
                      IIF(CloseFiles, "Files closed, ", ;
                                      IIF(lFullDetail,"Files not closed, ",""))+ ;
                      IIF(DefWinds,   "Windows defined, ", ;
                                      IIF(lFullDetail,"Windows not defined , ",""))+ ;
                      IIF(RelWinds,   "Windows released , ", ;
                                      IIF(lFullDetail,"Windows not released, ",""))+ ;
                      IIF(ReadCycle,  "READ CYCLE, ", ;
                                      IIF(lFullDetail,"Not READ CYCLE, ",""))+ ;
                      IIF(Modal,      "Modal, ", ;
                                      IIF(lFullDetail,"Not modal, ",""))+ ;
                      IIF(MultReads,  "Multiple READs, ", ;
                                      IIF(lFullDetail,"Not multiple READs, ",""))+ ;
                      IIF(NoLock,     "READ NOLOCK, ", ;
                                      IIF(lFullDetail,"Not READ NOLOCK, ",""))+ ;
                      IIF(SaveCode,   "Windows objects only, ", ;
                                      IIF(lFullDetail,"Cross platform objects, ",""))+ ;
                      IIF(DefName,    "Defaulted ", ;
                                      IIF(lFullDetail,"Not defaulted ",""))+ ;
                      IIF(EMPTY(AssocWinds),"",CHR(13)+"Associated Windows: "+ ;
                                               ALLTRIM(AssocWinds))
      CASE  Type = "s"
      CASE  Type = "x"
      CASE  Type = "Z"
      OTHERWISE
         cRetString = ""
   ENDCASE
ENDIF

* Remove any trailing commas

IF RIGHT(ALLTRIM(cRetString), 1) = ","
   cRetString = SUBSTR(ALLTRIM(cRetString), 1, LEN(ALLTRIM(cRetString))-1)
ENDIF

* Add the directory to the notes field if requested by the user.

IF lDirectory AND !EMPTY(m.mDirName)
   cRetString = "Directory is : "+ALLTRIM(m.mDirName) +IIF(_DOS OR _UNIX, " ", CHR(13))+cRetString
ENDIF

RETURN cRetString   



****************************************************************************
*
*                              ShortNameFN
*
*  This procedure strips off the relative directory of the file name.  
*  This is done for sorting purposes since FoxPro stores the directory
*  and the file name in the "name" field in the project file.
*
*  INPUT PARAMETERS:
*     None
*
*  OUTPUT PARAMETERS:
*     cRetString = The file name returned from the function
*
****************************************************************************

PROCEDURE ShortNameFN

PRIVATE cRetString                     &&  Value returned by the function

cRetString = UPPER(SUBSTR(name,RAT("\",name)+1))

* Eliminate box character in WIN preview

cRetString = STRTRAN(cRetString, CHR(0), "")       

RETURN  cRetString



****************************************************************************
*
*                              TStamp2DateFN
*
*  This procedure handles the conversion of the FoxPro TIMESTAMP field to 
*  a readable (and understandable) date and time.  The procedure will return
*  the date/time in three formats based on the cStyle parameter.  Timestamp 
*  field is a 32-bit (numeric compressed) system that the FoxPro development
*  team created to save on file space in the projects, screens, reports, and
*  label databases.  This field is used to determine if objects need to be 
*  recompiled (project manager), or syncronized across platforms (screens,
*  reports, and labels).
*
*  INPUT PARAMETERS:
*     nTimeStamp = Required field, must be numeric, no check to verify the
*                  data passed is valid FoxPro Timestamp, just be sure it is
*     cStyle     = Not required (defaults to "DATETIME"), must be character, 
*                  and must be one of the following:
*                   "DATETIME" will return the date/time in MM/DD/YY HH:MM:SS
*                   "DATE"     will return the date in MM/DD/YY format
*                   "TIME"     will return the time in HH:MM:SS format
*
*  OUTPUT PARAMETERS:
*     cRetval    = The date/time (in requested format) is returned in 
*                  character type.  Must be converted and parsed to be
*                  used as date type.
*
****************************************************************************

FUNCTION TStamp2DateFN
PARAMETER nTimeStamp, cStyle

PRIVATE cRetVal                        &&  Requested data returned from procedure

IF TYPE('nTimeStamp') != "N"           &&  Timestamp must be numeric
   WAIT WINDOW "Time stamp passed is not numeric"
   RETURN ""
ENDIF 

IF nTimeStamp = 0                      &&  Timestamp is zero until built in project
   RETURN "Not built into App"
ENDIF

IF TYPE('cStyle') != "C"               &&  Default return style to both date and time
   cStyle = "DATETIME"
ENDIF

IF !INLIST(UPPER(cStyle), "DATE", "TIME", "DATETIME")
   WAIT WINDOW "Style parameter must be DATE, TIME, or DATETIME"
ENDIF

nYear   = ((nTimeStamp/(2**25) + 1980))
nMonth  = ((nYear-INT(nYear)    )*(2**25))/(2**21)
nDay    = ((nMonth-INT(nMonth)  )*(2**21))/(2**16)

nHour   = ((nDay-INT(nDay)      )*(2**16))/(2**11)
nMinute = ((nHour-INT(nHour)    )*(2**11))/(2**05)
nSecond = ((nMinute-INT(nMinute))*(2**05))*2          &&  Multiply by two to correct 
                                                      &&  truncation problem built in
                                                      &&  to the creation algorithm
                                                      &&  (Source: Microsoft Tech Support)

cRetVal = ""
    
IF "DATE" $ UPPER(cStyle) 
   cRetVal = cRetVal + RIGHT("0"+ALLTRIM(STR(INT(nMonth))),2) + "/" + ;
                       RIGHT("0"+ALLTRIM(STR(INT(nDay))),2)   + "/" + ;
                       RIGHT("0"+ALLTRIM(STR(INT(nYear))),2)
ENDIF
   
IF "TIME" $ UPPER(cStyle)
   cRetVal = cRetVal + IIF("DATE" $ UPPER(cStyle), " ", "")
   cRetVal = cRetVal + RIGHT("0"+ALLTRIM(STR(INT(nHour))),2)   + ":" + ;
                       RIGHT("0"+ALLTRIM(STR(INT(nMinute))),2) + ":" + ;
                       RIGHT("0"+ALLTRIM(STR(INT(nSecond))),2)
ENDIF

RETURN cRetVal



****************************************************************************
*
*                              vSelProjPbPR
*
*  This procedure brings up the file selection dialog to pick the project.
*
*  INPUT PARAMETERS:
*     None
*
*  OUTPUT PARAMETERS:
*     None
*
****************************************************************************

PROCEDURE vSelProjPbPR

cProjDbf = GETFILE("PJX","Select project file","SELECT",0)

SHOW GET cProjDbf
   
IF !EMPTY(cProjDbf)
   _CUROBJ = OBJNUM(cOutputRb)
ENDIF

RETURN .T.



****************************************************************************
*
*                              vAboutPbPR
*
*  This procedure executes the About window for the application.
*
*  INPUT PARAMETERS:
*     None
*
*  OUTPUT PARAMETERS:
*     None
*
****************************************************************************

PROCEDURE vAboutPbPR

PRIVATE cSaveSelect

PUSH KEY CLEAR                         &&  Don't allow help screen to loop too many reads

cSaveSelect = SELECT()

IF !USED("plisthlp")
   USE plisthlp IN SELECT(1)           &&  Open if not previously
ENDIF

SELECT plisthlp                        &&  Select help file for application
cSaveRecNo = RECNO()                   &&  Saves current help record in case on About window

* Find field help in help database

LOCATE FOR UPPER(plisthlp.cField) = "APPLICATION"

* Display on screen if found, otherwise warn the user of developer flaw

IF EOF()
   WAIT WINDOW "No help defined for this application..." NOWAIT
ELSE
   SCATTER MEMVAR MEMO
   DO AbotPlts.spr
   
   IF cSaveRecNo > 0
      GO (cSaveRecNo)
   ENDIF
   
   SCATTER MEMVAR MEMO
ENDIF

* Clean up workarea pointers when done

SELECT (cSaveSelect)

POP KEY

RETURN



****************************************************************************
*
*                              vWhatsNewPbPR
*
*  This procedure executes the Whats new window for the application.
*
*  INPUT PARAMETERS:
*     None
*
*  OUTPUT PARAMETERS:
*     None
*
****************************************************************************

PROCEDURE vWhatsNewPbPR

DO HelpPR WITH "cWhatsNewPb "

RETURN



****************************************************************************
*
*                              vOutPutRbPR
*
*  This procedure sets the output destination variable (cOutput)
*  according to selection of the user.
*
*  INPUT PARAMETERS:
*     None
*
*  OUTPUT PARAMETERS:
*     None
*
****************************************************************************

PROCEDURE vOutPutRbPR

DO CASE
   CASE UPPER(cOutputRb) = "TO PRINTER"
      cOutput = "PRINTER"
   CASE UPPER(cOutputRb) = "TO FILE"
      cOutput = "FILE"
   CASE UPPER(cOutputRb) = "TO BROWSE"
      cOutput = "BROWSE"
   OTHERWISE
      cOutput = "SCREEN"
ENDCASE

RETURN



****************************************************************************
*
*                              vNotesRbPR
*
*  This procedure sets the lNotes and lFullDetail variables according to 
*  selection of the user.
*
*  INPUT PARAMETERS:
*     None
*
*  OUTPUT PARAMETERS:
*     None
*
****************************************************************************

PROCEDURE vNotesRbPR

lNotes      = IIF(UPPER(cNotesRb)="NO NOTES", .F., .T.)
lFullDetail = IIF(UPPER(cNotesRb)!="COMPLETE DETAILS", .F., .T.)

RETURN



****************************************************************************
*
*                              HelpPR
*
*  This procedure displays field level help to the user.
*
*  INPUT PARAMETERS:
*     cUserField = the current selected field variable name that 
*                  the user is on
*
*  OUTPUT PARAMETERS:
*     None
*
****************************************************************************

PROCEDURE HelpPR
PARAMETER cUserField

PRIVATE cSaveSelect

PUSH KEY CLEAR                         &&  Don't allow help screen to loop too many reads

cSaveSelect = SELECT()

IF !USED("plisthlp")
   USE plisthlp IN SELECT(1)           &&  Open if not previously
ENDIF

SELECT plisthlp                        &&  Select help file for application
cSaveRecNo = RECNO()                   &&  Saves current help record in case on About window

* Find field help in help database

LOCATE FOR UPPER(plisthlp.cField) = UPPER(cUserField)

* Display on screen if found, otherwise warn the user of developer flaw

IF EOF()
   WAIT WINDOW "No help defined for this field..." NOWAIT
ELSE
   SCATTER MEMVAR MEMO
   DO HelpPlts.spr
   
   IF cSaveRecNo > 0
      GO (cSaveRecNo)
   ENDIF
   
   SCATTER MEMVAR MEMO
ENDIF

* Cleanup workarea pointers when done

SELECT (cSaveSelect)

POP KEY

RETURN



****************************************************************************
*
*                              vPrintHlpPbPR
*
*  This procedure executes the Whats new window for the application.
*
*  INPUT PARAMETERS:
*     None
*
*  OUTPUT PARAMETERS:
*     None
*
****************************************************************************

PROCEDURE vPrintHlpPbPR

PRIVATE cSaveSelect

cSaveSelect = SELECT()

IF !USED("plisthlp")
   USE plisthlp IN SELECT(1)           &&  Open if not previously
ENDIF

SELECT *;
   FROM     Plisthlp;
   ORDER BY nPrintOrd;
   INTO CURSOR Query

IF _TALLY < 1
   WAIT WINDOW "No help available to print..."
ELSE
   DO CASE 
      CASE cOutput = "PRINTER"
         IF _DOS OR _UNIX
            WAIT WINDOW "Printing application documentation..." NOWAIT
         ENDIF   

         REPORT FORM HelpPltR TO PRINTER NOCONSOLE
   
      CASE cOutput = "FILE"
         cFileName = PUTFILE("Report file name","PJLHELP.TXT","TXT;REP","Select")

         IF !EMPTY(cFileName)
            cOldPdSetup = SET("PDSETUP")
            SET PDSETUP TO             &&  No print driver used for file output
         
            REPORT FORM HelpPltR ;
               TO FILE (cFileName) ;
               NOCONSOLE

            SET PDSETUP TO "&cOldPdSetup"
         ELSE
            WAIT WINDOW "No output file selected, documentation report not created" NOWAIT
         ENDIF

      OTHERWISE
         REPORT FORM HelpPltR PREVIEW
   ENDCASE
ENDIF

SELECT query
USE
 
* Cleanup workarea pointers when done

SELECT (cSaveSelect)

RETURN



****************************************************************************
*
*                              CleanupPR
*
*  This procedure cleans up the FoxPro environment and is run before
*  terminating the application.
*
*  INPUT PARAMETERS:
*     None
*
*  OUTPUT PARAMETERS:
*     None
*
****************************************************************************

PROCEDURE CleanupPR

* Check field type in case of exiting program before they are defined

IF TYPE('cOldTalk') = "C"
   SET TALK &cOldTalk
ENDIF

IF TYPE('cOldExclusive') = "C"
   SET EXCLUSIVE &cOldExclusive
ENDIF

IF TYPE('cOldDeleted') = "C"
   SET DELETED &cOldDeleted
ENDIF

IF WEXIST("wrParmPlts")
   RELEASE WINDOW wrParmPlts
ENDIF

POP KEY                                &&  Restore ON KEY settings

ON KEY LABEL F1 HELP                   &&  Restore FoxPro's help system
 

* Close only databases used by this program

IF USED("query") 
   SELECT query                          
   USE
ENDIF

IF USED("projlist")
   SELECT projlist
   USE
ENDIF

IF USED("projrep")
   SELECT projrep
   USE
ENDIF

IF USED("plisthlp")
   SELECT plisthlp
   USE
ENDIF

RETURN



*: EOF :*